#ifndef USERLIBSIMULATOR_H
#define USERLIBSIMULATOR_H

//! \file UserLibSimulator.h
//! \brief Archivo de cabecera para utilizar la librería de E/S con el simulador de Sistemas Físicos.
//!
//! Para compilación con QtCreator, añadir lo siguiente al .pro (cambiando el contenido de SIMUL_LIBRARY_PATH por el directorio donde se haya descomprimido la librería).<br>
//! Archivo .pro:<br>
//! \code{.unparsed}
//! SIMUL_LIBRARY_PATH = C:/User/.../Client/Library
//! INCLUDEPATH += $${SIMUL_LIBRARY_PATH}/include
//! contains(QT_ARCH, i386) {
//!     message("32-bit")
//!     LIBS += -L$${SIMUL_LIBRARY_PATH}/mingw32 -lUserLibSimulator
//! } else {
//!     message("64-bit")
//!     LIBS += -L$${SIMUL_LIBRARY_PATH}/mingw64 -lUserLibSimulator
//! }
//! # Si se desea utilizar curses:
//! LIBS += -lpdcurses
//! # Si se desea utilizar windows sockets:
//! LIBS += -lws2_32
//! \endcode
//! <br>
//! Ejemplo de uso<br>
//! test.c:<br>
//! \code{.c}
//! #include <stdio.h>
//! #include "UserLibSimulator.h"
//! int main()
//! {
//!    printf("Hello Simulator User!\n"); fflush(stdout);
//!    // Connect with simulator (must be running previously with Distrib/Client/Piston/SimuladorPiston.bat)
//!    int err=Simulator_ConnectWss("AlMarPiston","alumno","laclavemostradaenlapaginaweb","127.0.0.1",8080);
//!
//!    if (err!=CONNECT_IS_OK)
//!        return -1;
//!
//!    while (1)
//!    {
//!        int ad0,ad1;
//!        ad0=Simulator_ReadAD(0);
//!        ad1=Simulator_ReadAD(1);
//!        printf("AD0 = %d , AD1 = %d\n",ad0,ad1);
//!        Simulator_Delay(500);
//!    }
//!    return 0;
//! }
//! \endcode


#ifdef __cplusplus
extern "C" {
#endif

#define SIMULATOR_Vcc_V     5.0f    //!< Tensión de alimentación de las E/S del simulador (5V)

#define SIMULATOR_NO_ERROR  0   //!< No ha habido error en la última función ejecutada

//! Codigos de error de la función Simulator_ConnectWss()

#define CONNECT_IS_OK       SIMULATOR_NO_ERROR  //!< Conexión válida
#define CONNECT_ERROR_SERVER_NOT_RUNNING  -1    //!< El servidor MongooseWebServer.exe no se está ejecutando
#define CONNECT_ERROR_SERVER_NOT_RESPONDING  -2 //!< El servidor MongooseWebServer.exe no responde

#define N_MAX_TIMERS        4   //!< Máximo número de timers admitidos (usar 0 a 3)
#define N_MAX_AD_CONVERTERS 4   //!< Máximo número de ADC admitidos (usar 0 a 3)

//! Conectar con el simulador: ejecutar una sola vez en main con los datos siguientes (se pueden copiar desde
//!                            la página web pulsando el botón "Mostrar info clientes"
//! \param appName = nombre de la aplicación ("Piston", "Feedback", etc.)
//! \param userName = usuario autorizado para la apicación ("alumno")
//! \param passwd = clave para el usuario encriptada
//! \param  ip_address = dirección ip del servidor ("127.0.0.1" para servidor local)
//! \param  port = puerto del servidor (8080 para servidor local)
//! Ejemplo:     int err=Simulator_ConnectWss("Piston","alumno","laclaveencriptada","127.0.0.1",8080);
//!              if (err==CONNECT_IS_OK) { ... }
//! \return CONNECT_IS_OK=0 si todo ha ido bien, valor menor que 0 si ha habido algún problema
int Simulator_ConnectWss(const char* appName,const char* userName,const char* passwd,const char* ip_address,int port);

//! Generar un retardo en la aplicación
//! \param value_ms = tiempo de retardo en milisegundos
void Simulator_Delay(int value_ms);

//! Leer un valor analógico del simulador, nº bits (n) y límites de tensión (U<sub>min</sub>-U<sub>max</sub>) según aplicación
//! \param ad_channel: número de canal AD, empezando por 0. Canales disponibles según aplicación
//! \return valor leído del canal, entre 0 y 2<sup>n</sup>-1, según relación lineal con la tensión a la entrada
int Simulator_ReadAD(int ad_channel);

//! Leer el puerto de entradas digitales del simulador (bits 0 a 31, usos según aplicación)
//! \return valor leído del puerto DI (único) según la tensión de cada pin de entrada (0V&rarr;0, 5V&rarr;1)
int Simulator_ReadDI();

//! Leer un valor numérico del contador, nº bits (n) según aplicación
//! \param enc_channel: número de canal de encóder, empezando por 0. Canales disponibles según aplicación
//! \return valor leído del canal, entre -2<sup>n-1</sup> y 2<sup>n-1</sup>-1, según cuenta de pulsos
int Simulator_ReadCounter(int enc_channel);

//! Resetea a 0 el contador, nº bits (n) según aplicación
//! \param enc_channel: número de canal de encóder, empezando por 0. Canales disponibles según aplicación
//! \return 0 si no hay error
int Simulator_ResetCounter(int enc_channel);


//! Leer el puerto de salidas digitales del simulador (bits 0 a 31, usos según aplicación)
//! \return valor leído del puerto DO (único) según la tensión de cada pin de salida (0V&rarr;0, 5V&rarr;1)
int Simulator_ReadDO();

//! Escribir el puerto de salidas digitales del simulador (bits 0 a 31, usos según aplicación)
//! \param value: valor a escribir en el puerto DO (único), cada pin de salida tomará la tensión según el valor de su bit (0&rarr;0V, 1&rarr;5V)
void Simulator_WriteDO(int value);

//! Preparar una salida PWM
//! \param pwm_channel: número de canal PWM, empezando por 0. Canales disponibles según aplicación
//! \param count_max: Tpwm en número de pasos de contador
//! \param count_value_default: Ton inicial en número de pasos de contador
//! \param count_unit_us: tiempo por paso de contador en us
void Simulator_ConfigPWM(int pwm_channel,int count_max,int count_value_default,int count_unit_us);

//! Establecer el valor de una salida PWM
//! \param pwm_channel: número de canal PWM, empezando por 0. Canales disponibles según aplicación
//! \param count_value: Ton inicial en número de pasos de contador
void Simulator_SetPWMvalue(int pwm_channel,int count_value);

//! Inicalizar el display LCD de 2x24
//! \return 0 si correcto, o número negativo en caso de error (simulador no inicializado)
int Simulator_LCD_init();

//! Establecer la posición del cursor en el display LCD de 2x24
//! \param col: número de columna, de 1 a 24
//! \param row: número de fila, de 1 a 2
//! \return 0 si correcto, o número negativo en caso de error (simulador no inicializado, fuera de rango de display)
int Simulator_LCD_gotoxy(int col,int row);

//! Escribir en la posición actual del cursor en el display LCD de 2x24
//! \param fmt: cadena de formato, similar a printf()<br>
//!             caracteres especiales: '\f' = borra display<br>
//!                                    '\b' = retrocede cursor<br>
//!                                    '\t' = avanza cursor<br>
//!                                    '\n' = retorno de carro
//! \param ...: número variable de argumentos, según cadena de formato, similar a printf()
//! \return número de caracteres escritos, o número negativo en caso de error (LCD no inicializado)
int Simulator_LCD_printf(const char* fmt,...);


//! Establecer una rutina de servicio de interrupción temporizada
//! \param timer_number: número de timer, valores de 0 a 3
//! \param tm_ms: periodo de temporización en milisegundos
//! \param Fn: puntero a la función que se desea ejecutar de forma asíncrona cada vez que venza el temporizador, o NULL para detener el temporizador
//! \return: valor entero 0 &rarr; no hay error, <0 &rarr; error
//!
//! Ejemplo:<br>
//! \code
//!     void MyISR()
//!     {
//!        printf("INTERRUPT\n"); fflush(stdout);
//!     }
//!     ...
//!
//!     int main()
//!     {
//!        int err=Simulator_ConnectWss(...); // ver argumentos en la ayuda de esta función
//!        if (err==CONNECT_IS_OK) {
//!            Simulator_SetTimerInterrupt(0,1000,MyISR);
//!            printf("Pulse INTRO para terminar\n"); fflush(stdout);
//!            getchar();
//!        }
//!        return 0;
//!    }
//! \endcode
int Simulator_SetTimerInterrupt(int timer_number,int tm_ms,void (*Fn)());


//! Obtiene un caracter de la consola de texto asociada al simulador
//! \param blocking: si verdadero (distinto de cero), espera hasta que haya un carácter disponible
//! \return código ASCII del carácter introducido en la consola de texto del simulador, o 0 si no hay ningún carácter disponible
char Simulator_Console_GetChar(int blocking);

//! Obtiene una línea de texto de la consola asociada al simulador
//! \param blocking: si verdadero (distinto de cero), espera hasta que haya una línea completa disponible
//! \param textBuffer: puntero a tabla de caracteres con espacio suficiente para la cadena
//! \param textBufferSize: longitud de la tabla 'textBuffer'
//! \return entero que indica el número de caracteres que se han leído
int Simulator_Console_GetString(int blocking,char* textBuffer,int textBufferSize);


//! Escribe una línea de texto en la consola asociada al simulador
//! \param fmt: cadena de formato, similar a printf()<br>
//! \param ...: número variable de argumentos, según cadena de formato, similar a printf()
//! \return número de caracteres escritos, o número negativo en caso de error
int Simulator_Console_Printf(const char* fmt,...);

//! Espera una línea de texto en la consola asociada al simulador, y guarda datos en variables indicadas
//! \param fmt: cadena de formato, similar a scanf()<br>
//! \param ...: número variable de argumentos, según cadena de formato, similar a scanf()
//! \return número de valores correctamente obtenidos del texto, o número negativo en caso de error
int Simulator_Console_Scanf(const char* fmt,...);

#ifdef __cplusplus
}       // End extern
#endif


#endif // USERLIBSIMULATOR_H
